/*
 * Copyright 2010-2011 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.sdyc.ndmp.protobuf.serializer;


import java.io.*;
import java.io.Externalizable;

/**
 * Java Serialization Redis serializer.
 * Delegates to the default (Java based) serializer input Spring 3.
 *
 * @author ZhenQin
 */
public class ExternalSerializer implements Serializer<Externalizable> {


    /**
     * 序列化表格
     */
    private static final long serialVersionUID = 1L;


    protected final Class<Externalizable> objectClass;



    public ExternalSerializer(Class<Externalizable> t) {
        this.objectClass = t;
    }


    @Override
    public Externalizable deserialize(byte[] bytes) {
        try {
            ObjectInputStream in = new ObjectInputStream(new ByteArrayInputStream(bytes));
            Externalizable object = objectClass.newInstance();
            object.readExternal(in);
            in.close();
            return object;
        } catch (Exception ex) {
            throw new IllegalArgumentException("Cannot deserialize", ex);
        }
    }

    @Override
    public byte[] serialize(Externalizable object) {
        if (object == null) {
            return new byte[0];
        }
        try {
            ByteArrayOutputStream out = new ByteArrayOutputStream();
            ObjectOutputStream outputStream = new ObjectOutputStream(out);

            object.writeExternal(outputStream);

            outputStream.flush();
            outputStream.close();
            return out.toByteArray();
        } catch (Exception ex) {
            throw new IllegalArgumentException("Cannot serialize", ex);
        }
    }
}