/*
 * Copyright 2010-2011 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.sdyc.ndmp.protobuf.serializer;


import org.apache.hadoop.io.Writable;

import java.io.*;

/**
 * Java Serialization Redis serializer.
 * Delegates to the default (Java based) serializer input Spring 3.
 *
 * @author ZhenQin
 */
public class WritableSerializer implements Serializer<Writable> {



    /**
     * 序列化表格
     */
    private static final long serialVersionUID = 1L;


    protected final Class<Writable> objectClass;



    public WritableSerializer(Class<Writable> t) {
        this.objectClass = t;
    }


    @Override
    public Writable deserialize(byte[] bytes) {
        try {
            DataInputStream in = new DataInputStream(new ByteArrayInputStream(bytes));
            Writable object = objectClass.newInstance();
            object.readFields(in);
            in.close();
            return object;
        } catch (Exception ex) {
            throw new IllegalArgumentException("Cannot deserialize", ex);
        }
    }

    @Override
    public byte[] serialize(Writable object) {
        if (object == null) {
            return new byte[0];
        }
        try {
            ByteArrayOutputStream out = new ByteArrayOutputStream();
            DataOutputStream outputStream = new DataOutputStream(out);

            object.write(outputStream);

            outputStream.flush();
            outputStream.close();
            return out.toByteArray();
        } catch (Exception ex) {
            throw new IllegalArgumentException("Cannot serialize", ex);
        }
    }
}